package com.nisum.product.service;

import com.nisum.product.dto.ProductOffers;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.spark.sql.*;
import org.apache.spark.sql.streaming.OutputMode;
import org.apache.spark.sql.streaming.StreamingQuery;
import org.apache.spark.sql.streaming.StreamingQueryException;
import org.apache.spark.sql.streaming.Trigger;
import org.apache.spark.sql.types.*;
import org.apache.spark.util.CollectionAccumulator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.codec.multipart.FilePart;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeoutException;

@Slf4j
@Service
public class ProductsOfferService implements Serializable {




    public String readFromCsvAndPushToKafka(String topicName) throws TimeoutException, StreamingQueryException {


        String batchId= new SimpleDateFormat("HHmmssSSS").format(new Date());
        sparkSession().sqlContext().udf()
                .register("calculateOfferPrice",(Double price, Double discountPercentage)->
                        ((100-discountPercentage)*price)/100, DataTypes.DoubleType);

        StructField[] structFields= new StructField[4];
        structFields[0]= new StructField("id",DataTypes.StringType,true, Metadata.empty());
        structFields[1]= new StructField("name",DataTypes.StringType,true,Metadata.empty());
        structFields[2]= new StructField("description",DataTypes.StringType,true,Metadata.empty());
        structFields[3]= new StructField("price",DataTypes.DoubleType,true,Metadata.empty());

        StructType structType= new StructType(structFields);

        Dataset<Row> df=sparkSession().readStream()
                .format("csv")
                .option("header",true)
                .schema(structType)
                .load("src/main/resources/csv");

        Dataset<Row> rowTransformDf=df.withColumn("discountPercentage",functions.lit(10.50))
                .withColumn("offerPrice",functions.callUDF("calculateOfferPrice",
                        functions.col("price").cast(DataTypes.DoubleType)
                        ,functions.col("discountPercentage").cast(DataTypes.DoubleType)))
                .withColumn("uniqueId", functions.lit(UUID.randomUUID().toString()))
                .withColumn("batchId",functions.lit(batchId));


        rowTransformDf.selectExpr("CAST(batchId AS STRING) AS key", "to_json(struct(*)) AS value")
                .writeStream()
                .format("kafka")
                .trigger(Trigger.Once())
                .option("kafka.bootstrap.servers", "HYD-LAP-00484.corp.nisum.com:9092")
                .option("topic", topicName)
                .option("checkpointLocation", "src/main/resources/data/CheckPointLocation")
                .start()
                .awaitTermination(5000);

        return batchId;
    }



    public void readFromKafkaTransformIntoCSV(String batchId) throws TimeoutException, StreamingQueryException {


        Dataset<Row> df = sparkSession()
                .readStream()
                .format("kafka")
                .option("kafka.bootstrap.servers", "HYD-LAP-00484.corp.nisum.com:9092")
                .option("subscribe", "test-product3")
                .option("startingOffsets", "earliest")
                                .option("startingOffsets", "earliest")
                                .option("failOnDataLoss", "false").
                    option("kafka.con.group.id", "localStream")
                .load();

        StructType productOffers = Encoders.bean(ProductOffers.class).schema();

        Dataset<ProductOffers> offersDataset=df.selectExpr("CAST(value as String) as message")
                .select(functions.from_json(functions.col("message"),productOffers ).as("data"))
                .filter(functions.col("data.batchId").equalTo(batchId))
                .filter(functions.col("data.name").contains("Apple"))
                .filter(functions.col("data.offerPrice").geq(20000))
                .select("data.*")
                .coalesce(1)
                .as(Encoders.bean(ProductOffers.class));



        StreamingQuery streamingQuery=offersDataset
                .writeStream()
                .trigger(Trigger.Once())
                .format("csv")
                .option("checkpointLocation", "src/main/resources/data/checkpointLocation/StreamingJob")
                .option("path","src/main/resources/processed/"+batchId)
                .outputMode(OutputMode.Append())
                .start();

        streamingQuery.awaitTermination();


    }


    private SparkSession sparkSession()
    {
        return SparkSession
                .builder()
                .appName("Product info API")
                .master("local[1]")
                .getOrCreate();
    }



}
